"""

"""
import os
import subprocess
import sys
from importlib import find_loader, import_module
from time import time
from typing import Dict, List

import boto3
import click
import yaml

import datateer_cli.logging as log
from datateer_cli import deploy_flow
from datateer_cli.docs import docs as docs_commands
from datateer_cli.pipeline import settings as pipeline_settings


class DatateerContext:
    def __init__(self):
        self.config = None  # stores the config from .datateer/config.yml
        self.flow = None  # stores the Flow between import and deploy


@click.group()
@click.pass_context
def cli(ctx):
    ctx.ensure_object(DatateerContext)
    ctx.obj.config = pipeline_settings.load_config()
    for k, v in ctx.obj.config.items():
        os.environ[k.upper()] = str(v)


@cli.command()
@click.pass_context
def tester(ctx):
    print(ctx.obj.config["datateer_deploy_key_prefect_lib"])


@cli.command(help="Displays a simple hello message")
@click.option(
    "-t",
    "--text",
    default="world",
    help="Whatever you send in will get echoed back out to you",
)
def hello(text):
    click.echo(f"hello, {text}")


@cli.command(help="Display the version and exit")
def version():
    from datateer_cli import __version__

    print(__version__)


# region agent


@cli.group(help="For working with the Prefect agent that executes pipelines")
def agent():
    pass


# endregion

# region developer


@cli.group(help="Commands to help the Datateer engineers with projects")
def developer():
    pass


@developer.command(
    help="Creates virtual environments; intended to be used at the repo root"
)
@click.pass_context
@click.option(
    "-v",
    "--venv-root",
    default="venv",
    help="The name of the folder where you want your venvs",
)
@click.option(
    "-r",
    "--requirements-dir",
    default="requirements",
    help='The name of the directory that contains your requirements files. "requirements.txt" will be installed in the main venv at venv/ or the directory specified in --venv-root. "requirements-dev.txt" will also be installed at the venv root. All others named "requirements-<name>.txt will be installed in independent venvs at venv/<name>.',
)
def venvs(ctx, venv_root, requirements_dir):
    from datateer_cli.developer_setup import setup

    setup(venv_root, requirements_dir)


@developer.command(
    help="Initialize Terraform remote state. Should be run in the <client-code>-infrastructure repository root, because it modifies the main.tf file"
)
@click.option("-c", "--client-code", required=True)
@click.option("-r", "--aws-region", default="us-east-1")
@click.option("-p", "--aws-profile", required=True)
def init_remote_state(aws_profile, client_code, aws_region):
    os.environ["AWS_DEFAULT_REGION"] = aws_region
    os.environ["CLIENT_CODE"] = client_code
    from datateer_cli.terraform_util import configure_remote_state_backend

    configure_remote_state_backend(aws_profile)


# endregion

# region docs


@cli.group(help="For generating docs including erds and dbt docs")
def docs():
    pass


@docs.command(help="Deploys erd-diagrams, intended to be used at the root")
@click.pass_context
def generate_erd(searchdir):

    docs_commands.generate_erd(searchdir)


@docs.command(
    name="deploy",
    help="Deploys docs including erd-diagrams and dbt docs, intended to be used at the root",
)
@click.pass_context
@click.option(
    "-m", "--dbt-models-dir", default="dbt/models", help="The dbt models directory."
)
@click.option("-t", "--target", default="prod")
@click.option("-c", "--client-code", default=None)
def deploy_docs(ctx, dbt_models_dir, target, client_code):
    if client_code is None:
        client_code = os.getenv("CLIENT_CODE")
    if client_code is None:
        raise click.ClickException("Client code not found.")
    docs_commands.deploy_docs(dbt_models_dir, target, client_code)


# end region

# region pipeline
def import_pipeline(pipelines_dir="orchestration", name=None):
    try:
        sys.path.insert(1, os.getcwd())  # finds the pipeline here
        # todo: I would like to be able to avoid installing datateer into the project's requirements, and just run the global version. But if I do that,
        # then it cannot import correctly within the venv
        # sys.path.insert(1, os.path.abspath('venv/lib/python3.8/site-packages/')) # finds prefect here
        # sys.path.insert(1, os.path.abspath('venv/src/datateer-prefect')) # finds datateer.prefect here
        # print(sys.prefix)
        # print('-----', sys.path)
        path = f"pipeline_{name}" if name is not None else "pipeline"
        mod = import_module(f"{pipelines_dir}.{path}")
        return mod.flow
    except ModuleNotFoundError as ex:
        print(f"ERROR: Could not find the module you requested: {ex}.")
        print("Are you in the pipeline project's root directory?")
        print(
            'Have you activated the venv in the pipeline project? (Run "source venv/bin/activate")'
        )


@cli.group(help="For working with pipelines")
@click.pass_context
@click.option(
    "-n",
    "--name",
    default=None,
    help='The name of the pipeline. e.g. pipeline_<name>.py. If not provided, "pipeline.py" will be used as the pipeline file name',
)
@click.option(
    "-p",
    "--pipelines-dir",
    default="orchestration",
    help='The name of the directory that contains your pipeline files. The default is "orchestration"',
)
@click.option(
    "-e",
    "--environment",
    default="local",
    help="The name of the environment. Loads the pipeline in context of that environment, including environment variables",
    type=click.Choice(["local", "prod", "staging"], case_sensitive=False),
)
def pipeline(ctx, name, pipelines_dir, environment="local"):
    ctx.ensure_object(DatateerContext)
    os.environ["DATATEER_ENV"] = environment
    pipeline = import_pipeline(pipelines_dir=pipelines_dir, name=name)
    ctx.obj.flow = pipeline


@pipeline.command(
    help="push environment settings into secure storage so that it can be used at runtime."
)
@click.option(
    "-c",
    "--config-dir",
    default=".datateer",
    help="Relative path to the configuration directory. Defaults to .datateer",
)
@click.option(
    "-s",
    "--settings-file",
    help="Relative path to a settings file containing environment variables. Defaults to .env",
)
def push_settings(config_dir, settings_file=None):
    environment = os.getenv("DATATEER_ENV")
    settings_file = pipeline_settings.determine_settings_file(
        environment, settings_file
    )
    settings = pipeline_settings.load_settings(settings_file)
    pipeline_settings.push_settings(settings, environment)
    pipeline_settings.push_config(config_dir, environment)


@pipeline.command()
@click.option("-c", "--config-dir", default=".datateer")
def pull_config(config_dir):
    environment = os.environ["DATATEER_ENV"]
    pipeline_settings.pull_config(config_dir, environment)


@pipeline.command()
@click.pass_context
def debug(ctx):
    from pathlib import Path

    path = Path(__file__)
    print("This is the debug function for testing things out")
    print("parent: ", path.parent)
    print(ctx.obj.flow)


@pipeline.command(help="packages and deploys a pipeline")
@click.option("--debug", is_flag=True)
@click.pass_context
def deploy(ctx, debug):
    ctx.ensure_object(DatateerContext)
    environment = os.environ["DATATEER_ENV"]
    ctx.obj.flow.name += "-" + environment
    click.echo(f"Deploying flow {ctx.obj.flow.name} to the {environment} environment")
    deploy_flow.do_deploy(ctx.obj.flow, environment=environment, debug=debug)


@pipeline.command()
@click.pass_context
@click.option(
    "-p",
    "--aws-profile",
    help="The name of an AWS profile from the AWS credentials file. Optional.",
)
def run(ctx, aws_profile):
    ctx.ensure_object(DatateerContext)
    if aws_profile:
        os.environ["AWS_PROFILE"] = aws_profile
    try:
        ctx.obj.flow.run(run_on_schedule=False)
    except Exception as ex:
        if "Secrets Manager can't find the specified secret" in str(ex):
            log.error(
                f'Could not find the settings for environment {os.getenv("DATATEER_ENV")}. Have you run datateer push-config {os.getenv("DATATEER_ENV")}?'
            )
        else:
            raise


@pipeline.command()
@click.pass_context
def visualize(ctx):
    ctx.ensure_object(DatateerContext)
    ctx.obj.flow.visualize()


# endregion


if __name__ == "__main__":
    cli()



"""A pre-commit hook"""

import argparse
import glob
import os
import sys
from typing import List, Sequence

FORBIDDEN_EXTENSIONS = ["pem"]


def find_forbidden_files(
    forbidden_extensions: List[str], files_to_check: List[str] = None
) -> List[str]:
    print("11111111", forbidden_extensions)
    forbidden_files = []
    if not forbidden_extensions:
        print("No extensions provided")
        return []
    if not files_to_check or not len(files_to_check):
        print("No files to check")
        return []
    for file in files_to_check:
        for extension in forbidden_extensions:
            if file.lower().endswith(extension.lower()):
                forbidden_files.append(file)
    return forbidden_files


def main() -> int:
    """Returns exit code 0 if all good, or 1 if forbidden files were found"""
    parser = argparse.ArgumentParser()
    parser.add_argument("--extensions", nargs=1, default=FORBIDDEN_EXTENSIONS)
    parser.add_argument("filenames", nargs=argparse.REMAINDER)
    args = parser.parse_args()

    retval = 0
    print("Scanning for forbidden extensions", args.extensions)
    print()

    print("2222222", args)
    files = find_forbidden_files(args.extensions[0].split(","), args.filenames)
    for f in files:
        print(f"Forbidden file at {f}")
        retval = 1
    return retval


if __name__ == "__main__":
    exit(main())



import base64
import random
import string
import subprocess

import boto3

# import docker


REPOSITORY_LIFECYCLE_POLICY = """
{
    "rules": [
        {
            "rulePriority": 1,
            "description": "Keep only two images, expire all others",
            "selection": {
                "tagStatus": "any",
                "countType": "imageCountMoreThan",
                "countNumber": 2
            },
            "action": {
                "type": "expire"
            }
        }
    ]
}
"""


# def get_aws_client(role_arn, aws_region, service_name='ecr'):
# key, secret, token = get_aws_session(role_arn)
# client = boto3.client(service_name, aws_access_key_id=key,
#                       aws_secret_access_key=secret, aws_session_token=token, region_name=aws_region)
# return client


# def get_aws_session(role_arn):
#     """Creates an AWS session for the role_arn. Assumes the caller has permission to assume the role"""
#     sts_client = boto3.client('sts')
#     session_name = 'pipeline-deployment-' + \
#         ''.join(random.choices(string.ascii_uppercase + string.digits, k=7))
#     assumed_role_obj = sts_client.assume_role(
#         RoleArn=role_arn, RoleSessionName=session_name)
#     creds = assumed_role_obj['Credentials']
#     return creds['AccessKeyId'], creds['SecretAccessKey'], creds['SessionToken']


def registry_info():
    """Gets registry information for the client ECR registry. Returns tuple of the registry's username, password, and URL."""
    # ecr_client = get_aws_client(role_arn, aws_region, 'ecr')
    ecr_client = boto3.client("ecr")
    ecr_creds = ecr_client.get_authorization_token()["authorizationData"][0]
    ecr_username = "AWS"
    ecr_password = (
        base64.b64decode(ecr_creds["authorizationToken"])
        .replace(b"AWS:", b"")
        .decode("utf-8")
    )
    ecr_url = ecr_creds["proxyEndpoint"].lstrip("https://")
    return ecr_username, ecr_password, ecr_url


def docker_login():
    """Logs into the docker registry for the client ECR registry. Works even if no repositories have been created yet."""
    username, password, url = registry_info()

    # the python docker library doesn't save credentials locally, so unfortunately we call subprocess instead
    # docker_client = docker.from_env()
    # res = docker_client.login(
    #     username=username, password=password, registry=url)
    # return res
    subprocess.run(["docker", "login", "-u", username, "-p", password, url])


def ensure_repository_exists(repo_name):
    # client = get_aws_client(role_arn, aws_region, 'ecr')
    client = boto3.client("ecr")
    try:
        client.create_repository(repositoryName=repo_name)
        client.put_lifecycle_policy(
            repositoryName=repo_name, lifecyclePolicyText=REPOSITORY_LIFECYCLE_POLICY
        )
    except client.exceptions.RepositoryAlreadyExistsException:
        pass



# # This file helps to compute a version number in source trees obtained from
# # git-archive tarball (such as those provided by githubs download-from-tag
# # feature). Distribution tarballs (built by setup.py sdist) and build
# # directories (produced by setup.py build) will contain a much shorter file
# # that just contains the computed version number.

# # This file is released into the public domain. Generated by
# # versioneer-0.18 (https://github.com/warner/python-versioneer)

# """Git implementation of _version.py."""

# import errno
# import os
# import re
# import subprocess
# import sys


# def get_keywords():
#     """Get the keywords needed to look up the version information."""
#     # these strings will be replaced by git during git-archive.
#     # setup.py/versioneer.py will grep for the variable names, so they must
#     # each be defined on a line of their own. _version.py will just call
#     # get_keywords().
#     git_refnames = "$Format:%d$"
#     git_full = "$Format:%H$"
#     git_date = "$Format:%ci$"
#     keywords = {"refnames": git_refnames, "full": git_full, "date": git_date}
#     return keywords


# class VersioneerConfig:
#     """Container for Versioneer configuration parameters."""


# def get_config():
#     """Create, populate and return the VersioneerConfig() object."""
#     # these strings are filled in when 'setup.py versioneer' creates
#     # _version.py
#     cfg = VersioneerConfig()
#     cfg.VCS = "git"
#     cfg.style = "pep440"
#     cfg.tag_prefix = ""
#     cfg.parentdir_prefix = "None"
#     cfg.versionfile_source = "datateer/devops/_version.py"
#     cfg.verbose = False
#     return cfg


# class NotThisMethod(Exception):
#     """Exception raised if a method is not valid for the current scenario."""


# LONG_VERSION_PY = {}
# HANDLERS = {}


# def register_vcs_handler(vcs, method):  # decorator
#     """Decorator to mark a method as the handler for a particular VCS."""

#     def decorate(f):
#         """Store f in HANDLERS[vcs][method]."""
#         if vcs not in HANDLERS:
#             HANDLERS[vcs] = {}
#         HANDLERS[vcs][method] = f
#         return f

#     return decorate


# def run_command(commands, args, cwd=None, verbose=False, hide_stderr=False, env=None):
#     """Call the given command(s)."""
#     assert isinstance(commands, list)
#     p = None
#     for c in commands:
#         try:
#             dispcmd = str([c] + args)
#             # remember shell=False, so use git.cmd on windows, not just git
#             p = subprocess.Popen(
#                 [c] + args,
#                 cwd=cwd,
#                 env=env,
#                 stdout=subprocess.PIPE,
#                 stderr=(subprocess.PIPE if hide_stderr else None),
#             )
#             break
#         except EnvironmentError:
#             e = sys.exc_info()[1]
#             if e.errno == errno.ENOENT:
#                 continue
#             if verbose:
#                 print("unable to run %s" % dispcmd)
#                 print(e)
#             return None, None
#     else:
#         if verbose:
#             print("unable to find command, tried %s" % (commands,))
#         return None, None
#     stdout = p.communicate()[0].strip()
#     if sys.version_info[0] >= 3:
#         stdout = stdout.decode()
#     if p.returncode != 0:
#         if verbose:
#             print("unable to run %s (error)" % dispcmd)
#             print("stdout was %s" % stdout)
#         return None, p.returncode
#     return stdout, p.returncode


# def versions_from_parentdir(parentdir_prefix, root, verbose):
#     """Try to determine the version from the parent directory name.

#     Source tarballs conventionally unpack into a directory that includes both
#     the project name and a version string. We will also support searching up
#     two directory levels for an appropriately named parent directory
#     """
#     rootdirs = []

#     for i in range(3):
#         dirname = os.path.basename(root)
#         if dirname.startswith(parentdir_prefix):
#             return {
#                 "version": dirname[len(parentdir_prefix) :],
#                 "full-revisionid": None,
#                 "dirty": False,
#                 "error": None,
#                 "date": None,
#             }
#         else:
#             rootdirs.append(root)
#             root = os.path.dirname(root)  # up a level

#     if verbose:
#         print(
#             "Tried directories %s but none started with prefix %s"
#             % (str(rootdirs), parentdir_prefix)
#         )
#     raise NotThisMethod("rootdir doesn't start with parentdir_prefix")


# @register_vcs_handler("git", "get_keywords")
# def git_get_keywords(versionfile_abs):
#     """Extract version information from the given file."""
#     # the code embedded in _version.py can just fetch the value of these
#     # keywords. When used from setup.py, we don't want to import _version.py,
#     # so we do it with a regexp instead. This function is not used from
#     # _version.py.
#     keywords = {}
#     try:
#         f = open(versionfile_abs, "r")
#         for line in f.readlines():
#             if line.strip().startswith("git_refnames ="):
#                 mo = re.search(r'=\s*"(.*)"', line)
#                 if mo:
#                     keywords["refnames"] = mo.group(1)
#             if line.strip().startswith("git_full ="):
#                 mo = re.search(r'=\s*"(.*)"', line)
#                 if mo:
#                     keywords["full"] = mo.group(1)
#             if line.strip().startswith("git_date ="):
#                 mo = re.search(r'=\s*"(.*)"', line)
#                 if mo:
#                     keywords["date"] = mo.group(1)
#         f.close()
#     except EnvironmentError:
#         pass
#     return keywords


# @register_vcs_handler("git", "keywords")
# def git_versions_from_keywords(keywords, tag_prefix, verbose):
#     """Get version information from git keywords."""
#     if not keywords:
#         raise NotThisMethod("no keywords at all, weird")
#     date = keywords.get("date")
#     if date is not None:
#         # git-2.2.0 added "%cI", which expands to an ISO-8601 -compliant
#         # datestamp. However we prefer "%ci" (which expands to an "ISO-8601
#         # -like" string, which we must then edit to make compliant), because
#         # it's been around since git-1.5.3, and it's too difficult to
#         # discover which version we're using, or to work around using an
#         # older one.
#         date = date.strip().replace(" ", "T", 1).replace(" ", "", 1)
#     refnames = keywords["refnames"].strip()
#     if refnames.startswith("$Format"):
#         if verbose:
#             print("keywords are unexpanded, not using")
#         raise NotThisMethod("unexpanded keywords, not a git-archive tarball")
#     refs = set([r.strip() for r in refnames.strip("()").split(",")])
#     # starting in git-1.8.3, tags are listed as "tag: foo-1.0" instead of
#     # just "foo-1.0". If we see a "tag: " prefix, prefer those.
#     TAG = "tag: "
#     tags = set([r[len(TAG) :] for r in refs if r.startswith(TAG)])
#     if not tags:
#         # Either we're using git < 1.8.3, or there really are no tags. We use
#         # a heuristic: assume all version tags have a digit. The old git %d
#         # expansion behaves like git log --decorate=short and strips out the
#         # refs/heads/ and refs/tags/ prefixes that would let us distinguish
#         # between branches and tags. By ignoring refnames without digits, we
#         # filter out many common branch names like "release" and
#         # "stabilization", as well as "HEAD" and "master".
#         tags = set([r for r in refs if re.search(r"\d", r)])
#         if verbose:
#             print("discarding '%s', no digits" % ",".join(refs - tags))
#     if verbose:
#         print("likely tags: %s" % ",".join(sorted(tags)))
#     for ref in sorted(tags):
#         # sorting will prefer e.g. "2.0" over "2.0rc1"
#         if ref.startswith(tag_prefix):
#             r = ref[len(tag_prefix) :]
#             if verbose:
#                 print("picking %s" % r)
#             return {
#                 "version": r,
#                 "full-revisionid": keywords["full"].strip(),
#                 "dirty": False,
#                 "error": None,
#                 "date": date,
#             }
#     # no suitable tags, so version is "0+unknown", but full hex is still there
#     if verbose:
#         print("no suitable tags, using unknown + full revision id")
#     return {
#         "version": "0+unknown",
#         "full-revisionid": keywords["full"].strip(),
#         "dirty": False,
#         "error": "no suitable tags",
#         "date": None,
#     }


# @register_vcs_handler("git", "pieces_from_vcs")
# def git_pieces_from_vcs(tag_prefix, root, verbose, run_command=run_command):
#     """Get version from 'git describe' in the root of the source tree.

#     This only gets called if the git-archive 'subst' keywords were *not*
#     expanded, and _version.py hasn't already been rewritten with a short
#     version string, meaning we're inside a checked out source tree.
#     """
#     GITS = ["git"]
#     if sys.platform == "win32":
#         GITS = ["git.cmd", "git.exe"]

#     out, rc = run_command(GITS, ["rev-parse", "--git-dir"], cwd=root, hide_stderr=True)
#     if rc != 0:
#         if verbose:
#             print("Directory %s not under git control" % root)
#         raise NotThisMethod("'git rev-parse --git-dir' returned error")

#     # if there is a tag matching tag_prefix, this yields TAG-NUM-gHEX[-dirty]
#     # if there isn't one, this yields HEX[-dirty] (no NUM)
#     describe_out, rc = run_command(
#         GITS,
#         [
#             "describe",
#             "--tags",
#             "--dirty",
#             "--always",
#             "--long",
#             "--match",
#             "%s*" % tag_prefix,
#         ],
#         cwd=root,
#     )
#     # --long was added in git-1.5.5
#     if describe_out is None:
#         raise NotThisMethod("'git describe' failed")
#     describe_out = describe_out.strip()
#     full_out, rc = run_command(GITS, ["rev-parse", "HEAD"], cwd=root)
#     if full_out is None:
#         raise NotThisMethod("'git rev-parse' failed")
#     full_out = full_out.strip()

#     pieces = {}
#     pieces["long"] = full_out
#     pieces["short"] = full_out[:7]  # maybe improved later
#     pieces["error"] = None

#     # parse describe_out. It will be like TAG-NUM-gHEX[-dirty] or HEX[-dirty]
#     # TAG might have hyphens.
#     git_describe = describe_out

#     # look for -dirty suffix
#     dirty = git_describe.endswith("-dirty")
#     pieces["dirty"] = dirty
#     if dirty:
#         git_describe = git_describe[: git_describe.rindex("-dirty")]

#     # now we have TAG-NUM-gHEX or HEX

#     if "-" in git_describe:
#         # TAG-NUM-gHEX
#         mo = re.search(r"^(.+)-(\d+)-g([0-9a-f]+)$", git_describe)
#         if not mo:
#             # unparseable. Maybe git-describe is misbehaving?
#             pieces["error"] = "unable to parse git-describe output: '%s'" % describe_out
#             return pieces

#         # tag
#         full_tag = mo.group(1)
#         if not full_tag.startswith(tag_prefix):
#             if verbose:
#                 fmt = "tag '%s' doesn't start with prefix '%s'"
#                 print(fmt % (full_tag, tag_prefix))
#             pieces["error"] = "tag '%s' doesn't start with prefix '%s'" % (
#                 full_tag,
#                 tag_prefix,
#             )
#             return pieces
#         pieces["closest-tag"] = full_tag[len(tag_prefix) :]

#         # distance: number of commits since tag
#         pieces["distance"] = int(mo.group(2))

#         # commit: short hex revision ID
#         pieces["short"] = mo.group(3)

#     else:
#         # HEX: no tags
#         pieces["closest-tag"] = None
#         count_out, rc = run_command(GITS, ["rev-list", "HEAD", "--count"], cwd=root)
#         pieces["distance"] = int(count_out)  # total number of commits

#     # commit date: see ISO-8601 comment in git_versions_from_keywords()
#     date = run_command(GITS, ["show", "-s", "--format=%ci", "HEAD"], cwd=root)[
#         0
#     ].strip()
#     pieces["date"] = date.strip().replace(" ", "T", 1).replace(" ", "", 1)

#     return pieces


# def plus_or_dot(pieces):
#     """Return a + if we don't already have one, else return a ."""
#     if "+" in pieces.get("closest-tag", ""):
#         return "."
#     return "+"


# def render_pep440(pieces):
#     """Build up version string, with post-release "local version identifier".

#     Our goal: TAG[+DISTANCE.gHEX[.dirty]] . Note that if you
#     get a tagged build and then dirty it, you'll get TAG+0.gHEX.dirty

#     Exceptions:
#     1: no tags. git_describe was just HEX. 0+untagged.DISTANCE.gHEX[.dirty]
#     """
#     if pieces["closest-tag"]:
#         rendered = pieces["closest-tag"]
#         if pieces["distance"] or pieces["dirty"]:
#             rendered += plus_or_dot(pieces)
#             rendered += "%d.g%s" % (pieces["distance"], pieces["short"])
#             if pieces["dirty"]:
#                 rendered += ".dirty"
#     else:
#         # exception #1
#         rendered = "0+untagged.%d.g%s" % (pieces["distance"], pieces["short"])
#         if pieces["dirty"]:
#             rendered += ".dirty"
#     return rendered


# def render_pep440_pre(pieces):
#     """TAG[.post.devDISTANCE] -- No -dirty.

#     Exceptions:
#     1: no tags. 0.post.devDISTANCE
#     """
#     if pieces["closest-tag"]:
#         rendered = pieces["closest-tag"]
#         if pieces["distance"]:
#             rendered += ".post.dev%d" % pieces["distance"]
#     else:
#         # exception #1
#         rendered = "0.post.dev%d" % pieces["distance"]
#     return rendered


# def render_pep440_post(pieces):
#     """TAG[.postDISTANCE[.dev0]+gHEX] .

#     The ".dev0" means dirty. Note that .dev0 sorts backwards
#     (a dirty tree will appear "older" than the corresponding clean one),
#     but you shouldn't be releasing software with -dirty anyways.

#     Exceptions:
#     1: no tags. 0.postDISTANCE[.dev0]
#     """
#     if pieces["closest-tag"]:
#         rendered = pieces["closest-tag"]
#         if pieces["distance"] or pieces["dirty"]:
#             rendered += ".post%d" % pieces["distance"]
#             if pieces["dirty"]:
#                 rendered += ".dev0"
#             rendered += plus_or_dot(pieces)
#             rendered += "g%s" % pieces["short"]
#     else:
#         # exception #1
#         rendered = "0.post%d" % pieces["distance"]
#         if pieces["dirty"]:
#             rendered += ".dev0"
#         rendered += "+g%s" % pieces["short"]
#     return rendered


# def render_pep440_old(pieces):
#     """TAG[.postDISTANCE[.dev0]] .

#     The ".dev0" means dirty.

#     Eexceptions:
#     1: no tags. 0.postDISTANCE[.dev0]
#     """
#     if pieces["closest-tag"]:
#         rendered = pieces["closest-tag"]
#         if pieces["distance"] or pieces["dirty"]:
#             rendered += ".post%d" % pieces["distance"]
#             if pieces["dirty"]:
#                 rendered += ".dev0"
#     else:
#         # exception #1
#         rendered = "0.post%d" % pieces["distance"]
#         if pieces["dirty"]:
#             rendered += ".dev0"
#     return rendered


# def render_git_describe(pieces):
#     """TAG[-DISTANCE-gHEX][-dirty].

#     Like 'git describe --tags --dirty --always'.

#     Exceptions:
#     1: no tags. HEX[-dirty]  (note: no 'g' prefix)
#     """
#     if pieces["closest-tag"]:
#         rendered = pieces["closest-tag"]
#         if pieces["distance"]:
#             rendered += "-%d-g%s" % (pieces["distance"], pieces["short"])
#     else:
#         # exception #1
#         rendered = pieces["short"]
#     if pieces["dirty"]:
#         rendered += "-dirty"
#     return rendered


# def render_git_describe_long(pieces):
#     """TAG-DISTANCE-gHEX[-dirty].

#     Like 'git describe --tags --dirty --always -long'.
#     The distance/hash is unconditional.

#     Exceptions:
#     1: no tags. HEX[-dirty]  (note: no 'g' prefix)
#     """
#     if pieces["closest-tag"]:
#         rendered = pieces["closest-tag"]
#         rendered += "-%d-g%s" % (pieces["distance"], pieces["short"])
#     else:
#         # exception #1
#         rendered = pieces["short"]
#     if pieces["dirty"]:
#         rendered += "-dirty"
#     return rendered


# def render(pieces, style):
#     """Render the given version pieces into the requested style."""
#     if pieces["error"]:
#         return {
#             "version": "unknown",
#             "full-revisionid": pieces.get("long"),
#             "dirty": None,
#             "error": pieces["error"],
#             "date": None,
#         }

#     if not style or style == "default":
#         style = "pep440"  # the default

#     if style == "pep440":
#         rendered = render_pep440(pieces)
#     elif style == "pep440-pre":
#         rendered = render_pep440_pre(pieces)
#     elif style == "pep440-post":
#         rendered = render_pep440_post(pieces)
#     elif style == "pep440-old":
#         rendered = render_pep440_old(pieces)
#     elif style == "git-describe":
#         rendered = render_git_describe(pieces)
#     elif style == "git-describe-long":
#         rendered = render_git_describe_long(pieces)
#     else:
#         raise ValueError("unknown style '%s'" % style)

#     return {
#         "version": rendered,
#         "full-revisionid": pieces["long"],
#         "dirty": pieces["dirty"],
#         "error": None,
#         "date": pieces.get("date"),
#     }


# def get_versions():
#     """Get version information or return default if unable to do so."""
#     # I am in _version.py, which lives at ROOT/VERSIONFILE_SOURCE. If we have
#     # __file__, we can work backwards from there to the root. Some
#     # py2exe/bbfreeze/non-CPython implementations don't do __file__, in which
#     # case we can only use expanded keywords.

#     cfg = get_config()
#     verbose = cfg.verbose

#     try:
#         return git_versions_from_keywords(get_keywords(), cfg.tag_prefix, verbose)
#     except NotThisMethod:
#         pass

#     try:
#         root = os.path.realpath(__file__)
#         # versionfile_source is the relative path from the top of the source
#         # tree (where the .git directory might live) to this file. Invert
#         # this to find the root from __file__.
#         for i in cfg.versionfile_source.split("/"):
#             root = os.path.dirname(root)
#     except NameError:
#         return {
#             "version": "0+unknown",
#             "full-revisionid": None,
#             "dirty": None,
#             "error": "unable to find root of source tree",
#             "date": None,
#         }

#     try:
#         pieces = git_pieces_from_vcs(cfg.tag_prefix, root, verbose)
#         return render(pieces, cfg.style)
#     except NotThisMethod:
#         pass

#     try:
#         if cfg.parentdir_prefix:
#             return versions_from_parentdir(cfg.parentdir_prefix, root, verbose)
#     except NotThisMethod:
#         pass

#     return {
#         "version": "0+unknown",
#         "full-revisionid": None,
#         "dirty": None,
#         "error": "unable to compute version",
#         "date": None,
#     }



"""Run this script to create or refresh your developer environment

Installs a virtual environment with the main requirements at venv, one at venv/dbt, and one at venv/meltano. The root (venv) can be changed by passing an argument e.g. "python requirements/developer_setup.py my_venv_root"
"""
import glob
import os
import re
import shutil
import subprocess
import sys
import venv

import pip

# REQUIREMENTS_DIR = 'requirements'
# SUB_VENVS = ['', 'dbt', 'meltano']


def setup(venv_root="venv", requirements_dir="requirements"):
    """Installs virtual environments in the venv folder"""
    if os.path.exists(venv_root):
        print(f"Path {venv_root} already exists; deleting")
        remove_directory(venv_root)

    builder = venv.EnvBuilder(clear=True, with_pip=True)

    # install requirements.txt and requirements-dev.txt in the main venv
    print()
    print(f">>>> creating virtual environment at {venv_root} <<<<")
    builder.create(os.path.join(venv_root))
    pip_install(venv_root, "", ["pip", "setuptools", "wheel"])
    pip_install(venv_root, "", ["-e", "."])
    if os.path.exists(os.path.join(requirements_dir, "requirements-dev.txt")):
        pip_install_requirements(
            venv_root, "", os.path.join(requirements_dir, "requirements-dev.txt")
        )

    # get the remaining requirements files
    sub_venvs = list(
        set(glob.glob(os.path.join(requirements_dir, "requirements-*.txt")))
        - set(glob.glob(os.path.join(requirements_dir, "requirements-dev.txt*")))
    )
    for req_filename in sub_venvs:
        m = re.search("requirements-(.*).txt", req_filename)
        venv_name = m.group(1)

        print()
        print(f">>>> creating sub virtual environment {venv_name} at {venv_root} <<<<")
        builder.create(os.path.join(venv_root, venv_name))
        pip_install(venv_root, venv_name, ["pip", "setuptools", "wheel"])
        pip_install_requirements(venv_root, venv_name, req_filename)


def remove_directory(dir):
    if os.name == "nt":
        os.system(f'rmdir /S /Q "{dir}"')
    else:
        shutil.rmtree(dir)


def executable_path(venv_root, venv_name):
    if os.name == "nt":
        return os.path.join(venv_root, venv_name, "scripts/python.exe")
    else:
        return os.path.join(venv_root, venv_name, "bin/python")


def pip_install_requirements(venv_root, venv_name, requirements_file, verbosity=0):
    """pip install a requirements file"""
    print()
    print(f">>>> installing {requirements_file} into venv {venv_root}/{venv_name}")
    command = [
        executable_path(venv_root, venv_name),
        "-m",
        "pip",
        "install",
        "-r",
        requirements_file,
    ]
    # append verbosity flag if included. e.g. verbosity=3 results in "-vvv"
    if verbosity is not None and 0 < verbosity:
        vs = ["v" for level in range(2)]
        command.append("-" + str.join("", vs))
    subprocess.run(command)


def pip_install(venv_root, venv_name, package_names, upgrade=True):
    """pip install a set of packages"""
    command = [
        executable_path(venv_root, venv_name),
        "-m",
        "pip",
        "install",
    ] + package_names
    if upgrade:
        command.append("--upgrade")
    print(f"Running {command} in {os.getcwd()}")
    subprocess.run(command)


if __name__ == "__main__":
    if 1 < len(sys.argv):
        setup(sys.argv[0], sys.argv[1])
    else:
        setup()



import os
import pprint
import re

import yaml
from erd_python.parser import parse
from erd_python.render import render

searchdir = (
    r"./dbt/models"  # we search for .yml files in the /dbt/models folder in the project
)


def get_yml(searchdir):
    print(f"Searching for .yml files in: {searchdir}")
    yml_files = []
    count = 0
    for root, dirs, files in os.walk(searchdir):
        for name in files:
            (base, ext) = os.path.splitext(name)  # split base and extension
            if ext == ".yml":  # check the extension
                count += 1
                full_name = os.path.join(root, name)  # create full path
                yml_files.append(full_name)
                print(full_name)
    print(f"Total number of .yml files found: {count}")
    return yml_files


def get_model(searchdir):
    fct_tables = {}
    dim_tables = {}
    brg_tables = {}
    yml_files = get_yml(searchdir)
    for yml_file in yml_files:
        with open(yml_file) as f:
            core = yaml.load(f, Loader=yaml.FullLoader)
            if "models" not in core.keys():
                continue
            fct_models = [m for m in core["models"] if m["name"].startswith("fct_")]
            dim_models = [m for m in core["models"] if m["name"].startswith("dim_")]
            brg_models = [m for m in core["models"] if m["name"].startswith("brg_")]
            for model in fct_models:
                columns = model["columns"]
                relationships = []
                for c in columns:
                    tests = c.get("tests")
                    if tests is not None:
                        for t in tests:
                            if "relationships" in t:
                                related_table_name = re.search(
                                    r"ref\('(.*?)'\)", t["relationships"]["to"]
                                ).group(1)
                                relationships.append(
                                    {
                                        "table": related_table_name,
                                        "from_field": c["name"],
                                        "to_field": t["relationships"]["field"],
                                        # 'columns': [col['name'] for col in next(model for model in all_models if model['name'] == related_table_name)['columns']]
                                    }
                                )
                fct_tables[model["name"]] = {
                    "columns": [column["name"] for column in model["columns"]],
                    "relationships": relationships,
                }
            for model in brg_models:
                columns = model["columns"]
                relationships = []
                for c in columns:
                    tests = c.get("tests")
                    if tests is not None:
                        for t in tests:
                            if "relationships" in t:
                                related_table_name = re.search(
                                    r"ref\('(.*?)'\)", t["relationships"]["to"]
                                ).group(1)
                                relationships.append(
                                    {
                                        "table": related_table_name,
                                        "from_field": c["name"],
                                        "to_field": t["relationships"]["field"],
                                        # 'columns': [col['name'] for col in next(model for model in all_models if model['name'] == related_table_name)['columns']]
                                    }
                                )
                brg_tables[model["name"]] = {
                    "columns": [column["name"] for column in model["columns"]],
                    "relationships": relationships,
                }
            for model in dim_models:
                columns = model["columns"]
                dim_tables[model["name"]] = {
                    "columns": [column["name"] for column in model["columns"]]
                }

    return fct_tables, dim_tables, brg_tables


# flake8: noqa: C901
def generate(searchdir):
    fct_tables, dim_tables, brg_tables = get_model(searchdir)
    pp = pprint.PrettyPrinter(indent=2)
    # pp.pprint(fct_tables)

    if not os.path.exists("build"):
        os.makedirs("build")
    if not os.path.exists("build/erds"):
        os.makedirs("build/erds")
    if not os.path.exists("dbt/assets"):
        os.makedirs("dbt/assets/")
    if not os.path.exists("dbt/assets/erds"):
        os.makedirs("dbt/assets/erds")

    for name, metadata in fct_tables.items():
        print(name)
        fct_brg_tables = set()
        with open(f"build/erds/{name}.er", "w") as f:
            f.write("# The fact table\n")
            f.write(f"[{name}]\n")
            for c in metadata["columns"]:
                if f"{name[4:-1]}_id" == c:
                    f.write(f"*{c}\n")
                elif c.endswith("_id"):
                    f.write(f"+{c}\n")
                else:
                    f.write(f"{c}\n")
            f.write("\n# Related tables\n")

            for rel in metadata["relationships"]:
                if rel["table"].startswith("dim"):
                    dim_name = rel["table"]
                    if dim_name in dim_tables:
                        cols = dim_tables[dim_name]
                        f.write(f"[{dim_name}]\n")
                        for c in dim_tables[dim_name]["columns"]:
                            if f"{dim_name[4:-1]}_id" == c:
                                f.write(f"*{c}\n")
                            else:
                                f.write(f"{c}\n")
                elif rel["table"].startswith("brg"):
                    brg_name = rel["table"]
                    fct_brg_tables.add(brg_name)
                    if brg_name in brg_tables:
                        cols = brg_tables[brg_name]
                        f.write(f"[{brg_name}]\n")
                        for c in brg_tables[brg_name]["columns"]:
                            if f"{brg_name[4:-1]}_id" == c:
                                f.write(f"*{c}\n")
                            else:
                                f.write(f"{c}\n")
                        for r in brg_tables[brg_name]["relationships"]:
                            print(r)
                            brg_dim_name = r["table"]
                            if brg_dim_name in dim_tables:
                                cols = dim_tables[brg_dim_name]
                                f.write(f"[{brg_dim_name}]\n")
                            for c in dim_tables[brg_dim_name]["columns"]:
                                if f"{brg_dim_name[4:-1]}_id" == c:
                                    f.write(f"*{c}\n")
                                else:
                                    f.write(f"{c}\n")

                f.write("\n")

            f.write("\n# Relationships\n")
            for rel in metadata["relationships"]:
                if rel["table"].startswith("dim"):
                    f.write(
                        f"{name}:{rel['from_field']} *--1 {rel['table']}:{rel['to_field']}\n"
                    )
                elif rel["table"].startswith("brg"):
                    f.write(
                        f"{name}:{rel['from_field']} *--* {rel['table']}:{rel['to_field']}\n"
                    )
                print(rel)
            for table_name in fct_brg_tables:
                if table_name in brg_tables:
                    for rel in brg_tables[table_name]["relationships"]:
                        f.write(
                            f"{table_name}:{rel['from_field']} *--1 {rel['table']}:{rel['to_field']}\n"
                        )
                        print(rel)

        with open(f"build/erds/{name}.er") as f:
            data = f.read()
            objects = parse(data)
            render(objects, f"dbt/assets/erds/{name}.png", "png")
            render(objects, f"build/erds/{name}.dot", "dot")



import boto3


def get_efs_id_by_name(name: str):
    client = boto3.client("efs")
    list = client.describe_file_systems()["FileSystems"]
    for efs in list:
        if efs["Name"] == name:
            return efs["FileSystemId"]

    raise Exception(f"Could not find an EFS named {name}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['datateer_cli', 'datateer_cli.docs', 'datateer_cli.pipeline']

package_data = \
{'': ['*'], 'datateer_cli': ['flow/*', 'ssh/*']}

install_requires = \
['PyYAML>=5.3.1,<6.0.0',
 'boto3>=1.20.37,<2.0.0',
 'click>=7.0',
 'erd-python>=0.6.1,<0.7.0',
 'pathspec>=0.9.0,<0.10.0']

entry_points = \
{'console_scripts': ['datateer = datateer_cli.cli:cli']}

setup_kwargs = {
    'name': 'datateer-cli',
    'version': '0.7.3',
    'description': 'Datateer CLI to support devops and infrastructure',
    'long_description': None,
    'author': 'Datateer',
    'author_email': 'dev@datateer.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import glob
import logging
import os
import tempfile
from importlib import find_loader, import_module
from pathlib import Path
from shutil import copy, copytree, ignore_patterns
from typing import Dict

import boto3
import click
import pathspec
import yaml

from datateer_cli.efs import get_efs_id_by_name
from datateer_cli.pipeline import settings

DOCKERFILE_TEMPLATES_DIR = Path(__file__).parent.joinpath("flow")


def do_deploy(flow, environment="staging", debug=False):
    # expensive loads, so defer until now
    from prefect.client.client import Client
    from prefect.executors import LocalExecutor

    # from prefect.environments import LocalEnvironment
    from prefect.run_configs import ECSRun
    from prefect.storage import Docker

    dockerfile_path = copy(os.path.join(DOCKERFILE_TEMPLATES_DIR, "Dockerfile"), ".")
    copy(os.path.join(DOCKERFILE_TEMPLATES_DIR, ".dockerignore"), ".")

    if os.getenv("MELTANO_DATABASE_URI") is None:
        print("MELTANO_DATABASE_URI is None")
    else:
        print(
            f'MELTANO_DATABASE_URI is populated with value {os.getenv("MELTANO_DATABASE_URI")[0:8]}***'
        )

    flow.storage = Docker(
        registry_url=image_registry_url(),
        python_dependencies=["boto3"],
        dockerfile=dockerfile_path,
        image_tag="latest",
        env_vars={
            "DATATEER_ENV": environment,
            # "AWS_DEFAULT_REGION": os.environ["AWS_DEFAULT_REGION"],
            # "S3_PREFECT_RESULTS_BUCKET": os.environ["S3_PREFECT_RESULTS_BUCKET"],
        },
        build_kwargs={
            "buildargs": {
                # during the build process, the build script runs "meltano install." By default, this command runs against an internal Sqlite database
                # To make sure this runs against the production meltano database, we ensure this env var exists and contains the connection string
                "MELTANO_DATABASE_URI": os.getenv("MELTANO_DATABASE_URI"),
                "DATATEER_DEPLOY_KEY_PREFECT_LIB": os.environ[
                    "DATATEER_DEPLOY_KEY_PREFECT_LIB"
                ],
            }
        },
    )

    task_definition = {
        "networkMode": "awsvpc",
        "requiresCompatibilities": ["FARGATE"],
        "cpu": "1 vcpu",
        "memory": "8 GB",
        "executionRoleArn": f"arn:aws:iam::{os.environ['AWS_ACCOUNT_ID']}:role/ecsTaskExecutionRole",
        "taskRoleArn": f"arn:aws:iam::{os.environ['AWS_ACCOUNT_ID']}:role/ecsTaskRole",
        "containerDefinitions": [
            {
                "name": "flow",
                "mountPoints": [
                    {
                        "containerPath": "/datateer/pipeline-data",
                        "sourceVolume": "pipeline-efs",
                    }
                ]
                # "logConfiguration": { # this won't create the log group, so we can't reference it until we have some mechanism to create it
                #     "logDriver": "awslogs",
                #     "options": {
                #         "awslogs-group": f"/datateer/pipelines/{flow.name}",
                #         "awslogs-region": os.environ["AWS_DEFAULT_REGION"],
                #         "awslogs-stream-prefix": "prefect-agent",
                #     },
                # },
            }
        ],
        "volumes": [
            {
                "name": "pipeline-efs",
                "efsVolumeConfiguration": {
                    "fileSystemId": get_efs_id_by_name("datateer-pipeline-efs")
                },
            }
        ],
    }
    flow.run_config = ECSRun(
        # cpu="1 vcpu",  # 1 vcpu == 1 GB. https://docs.aws.amazon.com/AmazonECS/latest/developerguide/task_definition_parameters.html
        # memory="8 GB",
        task_definition=task_definition,
        env={
            "DATATEER_ENV": environment,
            "AWS_DEFAULT_REGION": os.environ["AWS_DEFAULT_REGION"],
            "S3_PREFECT_RESULTS_BUCKET": os.environ["S3_PREFECT_RESULTS_BUCKET"],
        },
        run_task_kwargs={
            "networkConfiguration": {
                "awsvpcConfiguration": {
                    "subnets": [
                        settings.get_parameter("/pipeline/subnet_1"),
                        settings.get_parameter("/pipeline/subnet_2"),
                    ],
                    "securityGroups": [
                        settings.get_parameter("/pipeline/security_group")
                    ],
                    "assignPublicIp": "DISABLED",
                }
            }
        },
    )

    flow.executor = LocalExecutor()

    try:
        register(flow, environment)
    except ValueError:
        Client().create_project(os.environ["CLIENT_CODE"])
        register(flow, environment)


def register(flow, environment):
    import datateer_cli.docker_util as d

    d.docker_login()
    d.ensure_repository_exists(flow.name)
    flow.register(
        project_name=os.environ["CLIENT_CODE"],
        # add the environment label and the client label to restrict which agents will run this
        labels=[environment, os.environ["CLIENT_CODE"]],
    )


def image_registry_url(include_https=False):
    """Builds a well formed docker registry URL

    Arguments:
        account_id {string} -- the AWS account ID
        region {string} -- the AWS region e.g. us-east-1 or eu-west-1

    Keyword Arguments:
        include_https {bool} -- True to include the https:// prefix, false to leave off (default: {False})

    Returns:
        string -- A well formed docker registry URL
    """
    account_id = boto3.client("sts").get_caller_identity()["Account"]
    registry = f'{account_id}.dkr.ecr.{os.getenv("AWS_DEFAULT_REGION")}.amazonaws.com'
    return f"https://{registry}" if include_https else registry


# def validate_environment(ctx, name, environment):
#     path = '.env' if environment == 'local' else f'.env.{environment}'
#     if not os.path.exists(path):
#         raise click.BadParameter(f'Could not locate an .env file at {path}')
#     return environment
